#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>

int left_deepest(int start, double *set);
int right_deepest(int start, double *set);
void get_double(double *val);
void get_positive_int(int *val);
void left_turn(int deepest, double interval, double *depth, double *angle);
void right_turn(int deepest, int ending, double interval, double *depth, double *angle);

int main()
{
	double interval, max_depth = 0, start, finish, inc;
	const double tangens = tan(3.14*7/180);
	int n, i, deepest;

	/*input block*/
	printf("Enter the interval between points: ");
	get_double(&interval);
	if (interval <= 0) {
		printf("Interval must be positive.\n");
		exit(3);
	}

	printf("Enter the number of points: ");
	get_positive_int(&n);

	printf("Enter the range to bottom: ");
	get_double(&inc);
	if (inc <= 0) {
		printf("A range to bottom must be positive\n");
		exit(1);
	}

	/*arrays containing angles and depts values*/
	double depth[n];
	double angle[n];

	for (i = 0; i < n; i++) {
		printf("Enter the depth in point %i: ", i+1);
		get_double(&depth[i]);
		if (depth[i] < 0) {
			printf("A depth can't be negative\n");
			exit(1);
		}

		depth[i] += inc;

		if (depth[i] >= max_depth) {
			deepest = i;
			max_depth = depth[i];
		}
	}

	/*these 2 functions are filling angles and depts arrays with values*/
	left_turn(deepest, interval, depth, angle);
	right_turn(deepest, n, interval, depth, angle);

	/* the points where machine enters and leaves the earth*/
	start = depth[0]/tangens;
	finish = depth[n]/tangens;

	/*output block*/
	printf("\nEntrance point is in %.4lf to Point 1. Angle: 7 degrees\n\n", start);

	for (i = 0; i < deepest; i++) {
		printf("Point %i:\n\tdepth:\t%.4lf\n\t", i+1, depth[i]);
		if (angle[i] != 0)
			printf("angle:\t%.4lf degrees, machine moves down\n", angle[i]);
		else
			printf("angle:\t%.4lf degrees, machine moves ih parallel the earth\n", angle[i]);
	}

	for (i = deepest; i < n; i++) {
		printf("Point %i:\n\tdepth:\t%.4lf\n\t", i+1, depth[i]);
		if (angle[i] != 0)
			printf("angle:\t%.4lf degrees, machine moves up\n", angle[i]);
		else
			printf("angle:\t%.4lf degrees, machine moves ih parallel the earth\n", angle[i]);
	}

	printf("\nLeaving point is in %.4lf from Point %i. Angle: 7 degrees\n", finish,n);
	printf("\nEnter 'E' or 'e' for exit\n\n");

	while (tolower(getchar())!= 'e') {}
	return 0;
}

/*function getting a double number*/
void get_double(double *val)
{
	int res;

	res = scanf( "%lf", val);
	if (res < 1) {
		printf("You entered some trash\n");
		exit(1);
	}
}

/*function getting a positive integer number*/
void get_positive_int(int *val)
{
	int res;

	res = scanf( "%i", val);
	if (res < 1) {
		printf("You entered some trash\n");
		exit(1);
	}

	if (*val <= 0) {
		printf("You entered negative number\n");
		exit(2);
	}
}

/*function returns the index of the deepest point
after the start index*/
int left_deepest(int start, double *set)
{
	int max = start, i;
	double max_val = 0;

	for (i = start-1; i >= 0; i--) {
		if (set[i] > max_val) {
			max = i;
			max_val = set[i];
		}
	}

	return max;
}

/*function returns the index of the deepest point
before the start index*/
int right_deepest(int start, double *set)
{
	int max = start, i;
	double max_val = 0;

	for (i = start+1; set[i]; i++) {
		if (set[i] > max_val) {
			max = i;
			max_val = set[i];
		}
	}
	
	return max;
}

/*function filling with values a part of depts
and angles arrays before the deepest point*/
void left_turn(int deepest, double interval, double *depth, double *angle)
{
	/*a part of control height difference*/
	const double dh0 = tan(3.14*7/180) * interval;

	/*current height difference between current
	deepest point and smaller deepest point*/
	double dh;
	int temp, i, warps;

	do {
		temp = left_deepest(deepest, depth);
		dh = depth[deepest] - depth[temp];
		warps = deepest - temp;

		if (dh < dh0*warps) {
			for (i = (deepest-1); i >= temp; i--) {
				angle[i] = 180*atan2(dh, interval * warps)/3.14;
				depth[i] = depth[deepest] - dh*(deepest - i)/warps;
			}
		} else {
			for (i = (deepest-1); i >= temp; i--) {
				angle[i] = 7;
				depth[i] = depth[deepest] - dh0*(deepest - i);
			}
		}

		deepest = temp;
	} while (deepest > 0);
}

/*function filling with values a part of depts and angles
arrays after the deepest point including this point*/
void right_turn(int deepest, int ending, double interval, double *depth, double *angle)
{
	/*a part of control height difference*/
	const double dh0 = tan(3.14*7/180) * interval;

	/*current height difference between current
	deepest point and smaller deepest point*/
	double dh;
	int temp, i, warps;

	do {
		temp = right_deepest(deepest, depth);
		dh = depth[deepest] - depth[temp];
		warps = temp - deepest;

		if (dh < dh0*warps) {
			for (i = deepest; i <= temp; i++) {
				angle[i] = 180*atan2(dh, interval * warps)/3.14;
				depth[i] = depth[deepest] - dh*(i - deepest)/warps;
			}
		} else {
			for (i = deepest; i <= temp; i++) {
				angle[i] = 7;
				depth[i] = depth[deepest] - dh0*(i - deepest);
			}
		}

		deepest = temp;
	} while (deepest < ending);
}
